/* LatVis
 * Copyright (C) 2002 Jan Outrata <outrataj@phoenix.inf.upol.cz>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

// class of the ordered set

#ifndef __ORDEREDSET_H__
#define __ORDEREDSET_H__

#include "element.h"
#include "undoredo.h"
#include <set>
#include <list>

typedef unsigned short OrderedSetId;
const OrderedSetId NoOrderedSet = 0;

class OrderData;
class Diagram_Graphic;

class LIBLATVIS_API OrderedSet {
 public:
  enum data_t {DATA_TREE, DATA_LIST, DATA_ARRAY};
 protected:
  struct UndoRedoType {
    set<Element> elements;
    OrderData *data;
    Diagram_Graphic *diagram;
    bool changed;
    bool saved;
    string filename;
  };
  UndoRedo<UndoRedoType *> undo_list;
  OrderedSetId id;
  set<Element> elements;
  OrderData *data;
  data_t datatype;
  Diagram_Graphic *diagram;
  bool changed;
  bool saved;
  string filename;
  
  UndoRedoType *getState();
  // returns: actual object state
  void setState(UndoRedoType *state);
  // sets the object state to state
  void getElementsFromElementIds(set<Element> &s, set<ElementId> ss);
  // returns: elements (in s) with id in ss
  void getElementIdsFromElements(set<ElementId> &s, set<Element> ss);
  // returns: element ids (in s) of elements in in ss
 public:
  OrderedSet(data_t data_type, OrderedSetId set_id = NoOrderedSet);
  ~OrderedSet();
  void clear();
  // clears the set - both data and diagram
  void save_UndoRedo();
  // saves the state to undo-redo list
  void removeLast_UndoRedo();
  // removes the last state from undo-redo list
  void undo();
  // does undo - changes the state to the previous one
  void redo();
  // does redo - changes the state to the next one
  data_t getDataType() { return datatype; }
  // returns: set data representation
  bool isSetChanged() { return changed; }
  // returns: if the set is changed
  void setChanged() { changed = true; saved = false; }
  // the set is changed
  void setUpdated() { changed = false; }
  // the set is updated
  string getFilename() { return filename; }
  // returns: name of the file from which the set was loaded or to which 
  // it will be saved
  void setFilename(string name) { filename = name; }
  // sets filename
  bool isFileSaved() { return saved; }
  // returns: if the set is saved
  void fileSaved() { saved = true; changed = false; }
  // set was saved
  
  Element *newElement(ElementId x, ElementName name, Vertex *v = NULL);
  // inserts new element with name name and vertex v (to the diagram)
  // if x != NoElement, sets element id to x
  // returns: new element, or NULL if an element with name x already 
  // exists
  bool rename(Element x, ElementName name);
  // renames element x to name name
  // returns: if successful, not if an element with name name already 
  // exists
  bool removeElement(Element x);
  // removes element x
  // returns: if successful (not)
  bool orderElements(Element x, Element y);
  // orders elements x, y such, that x is less than y
  // returns: if successful (not)
  bool removeOrder(Element x, Element y);
  // removes order of elements x, y
  // returns: if successful (not)
  bool less(Element x, Element y, bool transitivity = true);
  // if transitivity is false, doesn't iterate transitively
  // returns: if element x is less than element y
  void getElements(set<Element> &s);
  // returns: all elements in s
  void upperElements(set<Element> &s, Element x);
  // returns: all upper elements of element x in s
  void lowerElements(set<Element> &s, Element x);
  // returns: all lower elements of element x in s
  void minimalElements(set<Element> &s, set<Element> ss);
  // returns: minimal elements of the subset ss in s
  void maximalElements(set<Element> &s, set<Element> ss);
  // returns: maximal elements of the subset ss in s
  Element *leastElement(set<Element> ss);
  // returns: the least element of the subset ss, or NULL
  Element *greatestElement(set<Element> ss);
  // returns: the greatest element of the subset ss, or NULL
  void lowerCone(set<Element> &s, set<Element> ss);
  // returns: lower cone of the subset ss in s
  void upperCone(set<Element> &s, set<Element> ss);
  // returns: upper cone of the subset ss in s
  Element *infimum(set<Element> ss);
  // returns: infimum element of the subset ss, or NULL
  Element *supremum(set<Element> ss);
  // returns: supremum element of the subset ss, or NULL
  void getPaths(list<set<Element> > &l, Element x, Element y);
  // returns: the list of paths from element x to element y in l
  
  Element *getElementFromElementId(ElementId x);
  // returns: element with id x, or NULL
  Element *getElement(ElementName x);
  // returns: element corresponding to element name x, or NULL
  Diagram_Graphic *getDiagram() { return diagram; }
  // returns: the diagram

  // comparison function for STL
  bool operator < (const OrderedSet &s) const { return (id < s.id); }
  operator OrderedSetId () { return id; }
};

#endif
