/* LatVis
 * Copyright (C) 2002 Jan Outrata <outrataj@phoenix.inf.upol.cz>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include "orderdata_tree.h"
#include "orderdata_list.h"
#include "orderdata_array.h"
#include "leveldiagram.h"
#include "geometricdiagram.h"
#include "layerdiagram.h"
#include <sys/time.h>
#include <sys/resource.h>
#include <sys/types.h>
#include <unistd.h>
#include <set>
#include <stdio.h>

typedef unsigned int count_t;

OrderData_Tree data_tree;
OrderData_List data_list;
OrderData_Array data_array;

count_t n, d;
OrderData *data[3];
char *reps[3] = {"tree", "list", "array"};
struct timeval before, after;
long mem = 0;

long
print_stat(struct timeval *t1, struct timeval *t2, long count, long old_mem)
{
  char path[80], buf[10];
  FILE *f;
  long diff, size = 0, nr;


  diff = (t2->tv_sec - t1->tv_sec) * 1000000 + (t2->tv_usec - t1->tv_usec);
  if (count) diff /= count;
  sprintf(path, "/proc/%d/%s", getpid(), "statm");
  f = fopen(path, "r");
  if (f) {
    nr = fread(buf, 1, (sizeof buf) - 1, f);
    if (nr > 0 ) {
      buf[nr] = 0;
      sscanf(buf, "%ld", &size);
    }
    fclose(f);
  }
  size = (size << 2) - 632 - old_mem;
  printf("%li.%03li ms, %li kB\n", diff / 1000, diff % 1000, size);
  return size;
}

void
newElement()
{
  ElementId i;
  
  printf("\nnewElement (%s): ", reps[d]);
  fflush(stdout);
  gettimeofday(&before, NULL);
  for (i = 1; i <= n; i++)
    data[d]->newElement(i);
  gettimeofday(&after, NULL);
  mem = print_stat(&before, &after, n, 0);
}

void
orderElements()
{
  ElementId i, j;

  printf("\norderElements (%s): ", reps[d]);
  fflush(stdout);
  gettimeofday(&before, NULL);
  for (i = 1; i <= n; i++)
    for (j = i; j <= n; j++)
      data[d]->orderElements(i, j);
  gettimeofday(&after, NULL);
  mem = print_stat(&before, &after, (n * (n + 1)) / 2, mem);
}

void
less_()
{
  ElementId i, j;
  
  printf("\nless (%s): ", reps[d]);
  fflush(stdout);
  gettimeofday(&before, NULL);
  for (i = 1; i <= n; i++)
    for (j = 1; j <= n; j++)
      data[d]->less(i, j);
  gettimeofday(&after, NULL);
  mem = print_stat(&before, &after, n * n, mem);
}

void ops()
{
  set<ElementId> s, s1, s2;
  ElementId e, i;

  printf("\noperations (%s): ", reps[d]);
  fflush(stdout);
  data[d]->getElements(s);
  gettimeofday(&before, NULL);
  for (i = 1; i <= n; i++) {
    data[d]->upperElements(s1, i);
    s1.clear();
  }
  data[d]->maximalElements(s1, s);
  e = data[d]->greatestElement(s);
  s2.insert(e);
  data[d]->lowerCone(s1, s2);
  gettimeofday(&after, NULL);
  mem = print_stat(&before, &after, 1, mem);
}

void
level()
{
  Diagram *diag;
  Diagram::generationParams_t *diag_params;

  printf("\nLevelDiagram (%s): ", reps[d]);
  fflush(stdout);
  diag = new LevelDiagram(data[d]);
  diag_params = new LevelDiagram::generationParams_t;
  ((LevelDiagram::generationParams_t *)diag_params)->from_minimal = true;
  gettimeofday(&before, NULL);
  diag->generateDiagram(diag_params);
  gettimeofday(&after, NULL);
  mem = print_stat(&before, &after, 1, mem);
}

void
geom()
{
  Diagram *diag;
  Diagram::generationParams_t *diag_params;

  printf("\nGeometricDiagram (%s): ", reps[d]);
  fflush(stdout);
  diag = new GeometricDiagram(data[d]);
  diag_params = new GeometricDiagram::generationParams_t;
  ((GeometricDiagram::generationParams_t *)diag_params)->from_greatest = true;
  gettimeofday(&before, NULL);
  diag->generateDiagram(diag_params);
  gettimeofday(&after, NULL);
  mem = print_stat(&before, &after, 1, mem);
}

void
layer()
{
  Diagram *diag;
  Diagram::generationParams_t *diag_params;
  
  printf("\nLayerDiagram (%s): ", reps[d]);
  fflush(stdout);
  diag = new LayerDiagram(data[d]);
  diag_params = new LayerDiagram::generationParams_t;
  ((LayerDiagram::generationParams_t *)diag_params)->from_maximal = true;
  ((LayerDiagram::generationParams_t *)diag_params)->layering =
    LayerDiagram::LAYERING_COFFMAN_GRAHAM;
  ((LayerDiagram::generationParams_t *)diag_params)->cg_width = 100;
  ((LayerDiagram::generationParams_t *)diag_params)->
    dummy_chains_data_type = LayerDiagram::DUMMY_D_ARRAY;
  ((LayerDiagram::generationParams_t *)diag_params)->crossing_reduction =
    LayerDiagram::CROSSING_REDUCTION_ADJACENT_EXCHANGE;
  gettimeofday(&before, NULL);
  diag->generateDiagram(diag_params);
  gettimeofday(&after, NULL);
  mem = print_stat(&before, &after, 1, mem);
}

int
main(int argc, char *argv[])
{
  count_t test;

  if ((argc != 4) && (argc != 1)) {

    printf("\n\
Usage: perf d n test\n\
\n\
d - data representation:\n\
 tree - 0\n\
 list - 1\n\
 array - 2\n\
\n\
n - count of the elements\n\
\n\
test - ordered set or diagram operation:\n\
 newElement - 0\n\
 orderElements - 1\n\
 less - 2\n\
 operations (upperElements, maximalElements, greatestElement, lowerCone) - 3\n\
 LevelDiagram - 4\n\
 GeometricDiagram - 5\n\
 LayerDiagram - 6\n\
\n");

    return 1;
  }
  if (argc == 1) {
    d = 2; n = 100; test = 2;
  }
  else {
    d = *argv[1] - '0'; sscanf(argv[2], "%i", &n); test = *argv[3] - '0';
  }
  data[0] = &data_tree; data[1] = &data_list; data[2] = &data_array;
  printf("\nn = %i\n", n);
  switch (test) {
  case 0:
    newElement();
    break;
  case 1:
    newElement(); orderElements();
    break;
  case 2:
    newElement(); orderElements(); less_();
    break;
  case 3:
    newElement(); orderElements(); ops();
    break;
  case 4:
    newElement(); orderElements(); level();
    break;
  case 5:
    newElement(); orderElements(); geom();
    break;
  case 6:
    newElement(); orderElements(); layer();
    break;
  }
  printf("\n");
  return 0;
}
